#File: bpy_reconstruction.py
#Author: Andres Jesse Porfirio (www.andresjesse.com)
#Compatible with Blender 2.63a

#This is a bpy Script, must be executed from Blender.

import bpy,os
from math import radians
import re

#----------------common defs
def key_func(string):
	result = re.split(r'(\d+)', string)
	for i in range(1, len(result), 2):
		result[i] = int(result[i])
	return result
#----------------

#READ params_file to create the list of images used in the reconstruction
params_file = open('params_file','r')
line = params_file.readline()
ifolder = line.split(' ')[0]
ofolder = line.split(' ')[1]
params_file.close()

images = []

for i in os.listdir(ifolder):
    if os.path.isfile(ifolder+i):
        images.append(i)

images.sort(key=key_func)

#reconstruction with two images (front and side), keep the first and last images
images = [ ifolder+images[0],ifolder+images[-1] ]

print(images)

#Cut detail: how many subdivisions will have the plane?
CUT_DETAIL = 50
#Displace param: 10 units offset to create the hand silhouette
DISPLACE_STRENGHT = 10

#This function loads all images as textures to be used in the 3D reconstruction
def load_images():
    
    def load_texture(txfile):
        bpy.ops.image.open(filepath=txfile)
        img = bpy.data.images[bpy.path.basename(txfile)]
        cTex = bpy.data.textures.new(img.name, type = 'IMAGE')
        cTex.image = img
    
    current_dir = bpy.data.filepath[:-len(bpy.path.basename(bpy.data.filepath))]
    
    for img in images:
        if not bpy.path.basename(img) in bpy.data.textures:
            load_texture(current_dir+img)
            print('Loaded Img:'+bpy.data.textures[-1].name)
        else:
            print('Img:'+img+' already loaded, skipping...')
        
    print(bpy.data.textures.values())

#Create the base mesh to be sculpted
def add_cube():
    bpy.ops.mesh.primitive_cube_add()
    bpy.context.object.name = 'main_object'
    return bpy.context.object

#Create a plane to be used as boolean operator
#1) Subdivide
#2) Apply modifier to create hightmap
def create_crop_plane(angle,image_index):
    bpy.ops.mesh.primitive_plane_add(rotation=angle)
    plane = bpy.context.object
    plane.name = 'plane_'+images[image_index]
    
    bpy.ops.object.mode_set(mode='EDIT') 
    bpy.ops.mesh.subdivide(number_cuts=CUT_DETAIL)
    bpy.ops.object.mode_set(mode='OBJECT') 
    
    mod = plane.modifiers.new(name='disp_'+images[image_index],type='DISPLACE')
    mod.texture=bpy.data.textures.get(bpy.path.basename(images[image_index]))
    mod.strength=10
    
    bpy.ops.object.modifier_apply(modifier='disp_'+images[image_index])
    
    return plane

#This function uses the given images to create the final mesh
#by using the silhouette reconstruction technique
def apply_booleans(main_mesh):
    angles = [ i*(90/float(len(images)-1)) for i in range(0,len(images)) ]
    print('Cut Angles:',angles)
    for angle in angles:
        #create the plane with heightmap based on the image
        plane = create_crop_plane( (0,radians(angle),0) ,angles.index(angle))
        
        #create boolean modifier
        mod = main_mesh.modifiers.new(name='bool_'+images[angles.index(angle)],type='BOOLEAN')
        mod.object = plane
        
        bpy.context.scene.objects.active = main_mesh
        bpy.ops.object.modifier_apply(modifier='bool_'+images[angles.index(angle)])
        
        bpy.context.scene.objects.active = plane
        bpy.ops.object.delete()

#Smooth
def post_process_mesh(main_obj):
    mod = main_obj.modifiers.new(name='hremesh', type='REMESH')
    mod.mode = 'SMOOTH'
    mod.octree_depth = 5
    mod.scale = 0.9 #default
    
    bpy.context.scene.objects.active = main_obj
    bpy.ops.object.modifier_apply(modifier='hremesh')

#Export the mesh as ply
def export_ply(ply_file):
    bpy.ops.export_mesh.ply(filepath=ply_file)

load_images()
main_obj = add_cube()
apply_booleans(main_obj)
post_process_mesh(main_obj)
export_ply(ofolder+'PLY/'+ifolder.split('/')[1]+'.ply')
bpy.ops.wm.save_as_mainfile(filepath=ofolder+'BLEND/'+ifolder.split('/')[1]+'.blend')

print('DONE:'+ofolder+ifolder.split('/')[1]+'.ply')