#File: lb_test_template_matching.py
#Author: Andres Jesse Porfirio (www.andresjesse.com)
#Date: 12/24/12
#requires LibSVM version 3.14

#This program does: 
#	1 - Conversion of the *.sig files to the libsvm format (output "sigs_svm_input" is used as input for libsvm)
#	2 - Separation of the "sigs_svm_input" database with 70% for train ("train.svm") and 30% for test ("test.svm")

import sys,os
import struct

class TestSVM:
	def __init__(self):
		self.SIGS_DIR = os.path.realpath('T2i/SVM')+'/'
		self.LIBSVM_DIR = os.path.realpath('libsvm')+'/'

		self.NUMBER_OF_EXECUTIONS = 10
		self.RANKING_TO_TEST = [1,3,5]

		#create folders to store the svm tests
		self.create_folders()

		#read *.sig and create a single svm-format file (sigs_svm_input), one signature per line.
		self.convert_sig_to_svm_format()

		#populate the sets with train and test files
		self.split_database()

		#realize the svm test for each set (this process is a little long, take a coffee...)
		self.execute_test()

		#generate logs and confusion matrix
		self.generate_logs()


	#----------------utils
	def clean_conf_name(self,fullname):
		cname = fullname.split('.')[0]
		return cname.split('_')[2]
	#----------------

	def create_folders(self):
		#create folders to store the svm tests
		if not os.path.exists('TESTS'): os.makedirs('TESTS')
		if not os.path.exists('TESTS/SVM'): 
			os.makedirs('TESTS/SVM')
		else:
			print('Folder TESTS/SVM already exists (delete it to execute a new test), aborting..')

	def read_signature(self,file_sig):
		with open(file_sig, "rb") as f:
			dimension_x = struct.unpack('i', f.read(4))[0]
			dimension_y = struct.unpack('i', f.read(4))[0]

			signature = []
			
			for i in range(dimension_x*dimension_y):
				signature.append(struct.unpack('f', f.read(4))[0])
			
			return dimension_x, dimension_y, signature

	#this method converts all .sigs into a single file in svm format
	#the output file must be splitted for train/test
	def convert_sig_to_svm_format(self,filename_out='sigs_svm_input'):
		f = open(filename_out,'w')

		for hcfg in os.listdir(self.SIGS_DIR):
			for fsig in os.listdir(self.SIGS_DIR+hcfg):
				if not fsig[-4:] == '.sig':
					continue

				dimension_x, dimension_y, signature = self.read_signature(self.SIGS_DIR+hcfg+'/'+fsig)

				svm_line = self.clean_conf_name(fsig)

				for i in range(len(signature)):
					svm_line += ' '+str(i+1)+':'+str(signature[i])

				f.write(svm_line+'\n')

		f.close()

	#This method splits the database ("sigs_svm_input") into train.svm and test.svm N times (instances randomly selected)
	def split_database(self,train_rate=0.7):
		num_train_instances = train_rate*610

		for i in range(self.NUMBER_OF_EXECUTIONS):
			os.makedirs('TESTS/SVM/SET'+str(i))
			cmd = 'python libsvm/tools/subset.py sigs_svm_input '+str(int(num_train_instances))+' TESTS/SVM/SET'+str(i)+'/train.svm TESTS/SVM/SET'+str(i)+'/test.svm'
			os.system(cmd)

	def execute_test(self):
		for i in range(self.NUMBER_OF_EXECUTIONS):

			print 'Executing test: SVM '+str(i+1)+' of '+str(self.NUMBER_OF_EXECUTIONS)
			set_folder = os.path.realpath('TESTS/SVM/SET'+str(i))+'/'

			#change current dir to the actual 'set' folder
			old_dir = os.getcwd()
			os.chdir(set_folder)

			#execute svm scripts
			cmd = 'python '+self.LIBSVM_DIR+'tools/execute.py train.svm test.svm '+self.LIBSVM_DIR
			os.system(cmd)

			#return to the original folder
			os.chdir(old_dir)

	#the log is generated based on the predicted file and the correct class from input file
	def generate_logs(self):
		print os.getcwd()

		for i in range(self.NUMBER_OF_EXECUTIONS):
			print 'Generating SVM log: '+str(i+1)+' of '+str(self.NUMBER_OF_EXECUTIONS)

			for rank in self.RANKING_TO_TEST:
				predict_file = 'TESTS/SVM/SET'+str(i)+'/test.svm.predict'
				input_test = 'TESTS/SVM/SET'+str(i)+'/test.svm'
				log_file = 'TESTS/SVM/SET'+str(i)+'/log_rank'+str(rank)+'.txt'

				cmd = 'python lb_svm_generate_log.py '+predict_file+' '+input_test+' '+str(rank)+' '+log_file
				os.system(cmd)

if __name__=='__main__':
	TestSVM()